// For use with Scilab 
// Copyright (C) 2017 -- Jérôme Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Luís Osório <lbica@hotmail.com>, Rui Araújo <rui@isr.uc.pt>
// Jérôme Mendes, Luís Osório, Rui Araújo, "The self-tuning pid controllers toolbox", 2017.
// The "The self-tuning pid controllers toolbox" comes with ABSOLUTELY NO WARRANTY;
// In case of publication of any application of these methods,
// please, cite the work:
// Jérôme Mendes, Luís Osório, and Rui Araújo. 
// Self-tuning pid controllers in pursuit of plug and play capacity. 
// Control Engineering Practice, 69:73-84, December 2017.
// DOI: https://doi.org/10.1016/j.conengprac.2017.09.006
// http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/


clc;
clear;
close;

// path of the code files
getd('//media/japm/DATA/Dropbox/Work/PhD/Projectos/SCIAD/SCIAD_LuisBica/My Scilab Files/Adaptive_PID/Clean') // Define your path where are located your code files

///////////////////////   Choose the controller algorithm (controller) and identification method  (ID_method) /////////////////////// 

//'1' or 'Dahlin'                  
//'3' or 'Pole Placement'          
//'4' or 'Deadbeat 2nd'            
//'5' or 'Deadbeat 3rd'            
//'6' or 'Ziegler Nichols FR'      
//'7' or 'Single Neuron        --> self idendification
//'8' or 'LSSVR'               --> self idendification
//'9' or 'LSSVRKT'             --> self idendification
controller = '1' // Defined by the user    

//     'LSM'     -> pure least squares method
//     'LSM_ef'  -> LSM with exponential forgetting 
//     'LSM_adf' -> LSM with adaptive directional forgetting
ID_method = 'LSM_adf' // Defined by the user    

///////////////////////  Controller Cicle Parameters    /////////////////////// 

// Define the maximum (Max_u), minimum(Min_u), and variation (Max_u_var) of the command signal:
Max_u = 1 //Select "Max_u=-1" to uncap the signal amplitude  // Defined by the user          
Min_u = 0;  //If Max_u==-1 then Min_u is ignored // Defined by the user    
Max_u_var = -1; //Select "Max_u_var=-1" to uncap the signal variation // Defined by the user    

// Cicle Parameters:
N = 1500;// Total number of time samples for experiments // Defined by the user    
T0 = 1;    //Sampling Period // Defined by the user    

na_max = 3; // Maximum order of the output of the model to be used on any controller // Defined by the user    
nb_max = 3; // Maximum order of the input of the model to be used on any controller  // Defined by the user    
ki = na_max + nb_max     //Dead time of the controller


///////////////////////  Definition of the reference (r)  /////////////////////// 

[r] = Reference(N);// Defined by the user as intended // Defined by the user    

///////////////////////  Initialization of the model/controller parameters   /////////////////////// 

iniValue = 0.0001; // initial value of the model parameters // Defined by the user    
          
[y,u,e,c,PHI,rho,phi,up,lamb,ep,eps,ye,X,i,w,vx,I_mU,sig,theta,na,nb,x] = initialize(ki,controller,N, iniValue);


///////////////////////////  Controller Cicle - Begin   ////////////////////////////////////////////
for k = ki:1:N

    e(k-1) =r(k) - y(k-1); // error clac
    disp( e(k-1))// Display error

    // To obtain the model parameters (if exists)
    if controller ~= 'Single Neuron' & controller ~= '7' & controller ~='LSSVR' & controller ~= '8' & controller ~= 'LSSVRKT' & controller ~= '9' then

        [theta,ep,eps,ye,c,PHI,rho,phi,up,lamb, G] = identify(na,nb,theta,ye,ep,eps,c,PHI,rho,phi,up,lamb,u,y,k,ki,ID_method);

    else
        theta(:,k)= theta(:,k-1);
    end

    // To obtain the control command signal
    [u,na,nb,theta,X,i,w,vx,I_mU,ye,ep,sig] = Tuning_And_PID(theta,T0,e,u,y,r,k,ki,X,i,w,vx,ye,ep,I_mU,sig,Max_u,Min_u,Max_u_var,controller);


    // Send the command signal to simulated plant
    [y(k)]=SimulatedPlant(y,u,k)// this plant it is here just as an example, we did not test these controllers in this plant. If you want use this plant with these controller you must tune the parameters that are defined by the user.

end

///////////////////////////  Controller Cicle - End   ////////////////////////////////////////////

// Final plots
figure();
plot(y);
figure();
plot(u);
figure();
plot(theta')
