// For use with Scilab 
// Copyright (C) 2017 -- Jérôme Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Luís Osório <lbica@hotmail.com>, Rui Araújo <rui@isr.uc.pt>
// Jérôme Mendes, Luís Osório, Rui Araújo, "The self-tuning pid controllers toolbox", 2017.
// The "The self-tuning pid controllers toolbox" comes with ABSOLUTELY NO WARRANTY;
// In case of publication of any application of these methods,
// please, cite the work:
// Jérôme Mendes, Luís Osório, and Rui Araújo. 
// Self-tuning pid controllers in pursuit of plug and play capacity. 
// Control Engineering Practice, 69:73-84, December 2017.
// DOI: https://doi.org/10.1016/j.conengprac.2017.09.006
// http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/
function [theta,ep,eps,ye,c,PHI,rho,phi,up,lamb, G]=identify(na,nb,theta,ye,ep,eps,c,PHI,rho,phi,up,lamb,u,y,k,ki,method)

    if k==ki then
        G = 1E-5 //initial covariance matrix value - Defined by the user
                    
        c = G*eye(na + nb , na + nb);  // initial covariance matrix
    end

    // auxiliar code
    PHI(1:na+nb,k) = [-y(k-2:-1:k-na-1);u(k-2:-1:k-nb-1)];

    ye(k) = theta(:,k-1)'*PHI(:,k);

    ep(k) = y(k-1)-theta(:,k-1)'*PHI(:,k);
    eps(k) = PHI(:,k)'*c*PHI(:,k);


/////////////////// LSM -> Least squares method ///////////////////
    if (method == 'LSM') then
      
        theta(:,k) = theta(:,k-1) + (c*PHI(:,k)*ep(k))/(1+eps(k)); // theta (model parameters) update
        c = c-(c*PHI(:,k)*PHI(:,k)'*c)/(1+eps(k)); // new covariance matrix
    

///////////////////  LSM_ef -> LSM with exponential forgetting ///////////////////
    elseif method == 'LSM_ef' then
        
        phi=0.999; //  0 < phi <= 1  - Defined by the user
        
        theta(:,k) = theta(:,k-1) + (c*PHI(:,k)*ep(k))/(phi+eps(k)); // theta (model parameters) update
        c =(1/phi)*(c-(c*PHI(:,k)*PHI(:,k)'*c)/(phi+eps(k))); // new covariance matrix

///////////////////  LSM_adf -> LSM with adaptive directional forgetting ///////////////////
    elseif method == 'LSM_adf' then

        if k==ki then
	   // learning parameters
           rho = 0.999; // Defined by the user       
           phi(k-1) = 0.001; // Defined by the user           
           up(k-1) = 0.1;  // Defined by the user    
           lamb(k-1) = 0.001;  // Defined by the user    
        end
              
        theta(:,k) = theta(:,k-1) + (c*PHI(:,k)*ep(k))/(1+eps(k)); // theta (model parameters) update

        if  eps(k)>0 then
            E = phi(k-1)-(1-phi(k-1))/eps(k);
            c = c-(c*PHI(:,k)*PHI(:,k)'*c)/(inv(E)+eps(k)); // new covariance matrix
            
        end
        if eps(k) >= 0 then
            lamb(k) = phi(k-1)*(lamb(k-1)+ep(k)*ep(k)/(1+eps(k)));
            up(k) = phi(k-1)*(up(k-1)+1);
            nu=ep(k)*ep(k)/lamb(k);
            phi(k) = 1/(1+(1+rho)*(log(1+eps(k))+((up(k)+1)*nu/(1+eps(k)+nu)-1)*eps(k)/(1+eps(k))));
        else
            lamb(k)=lamb(k-1);
            up(k)=up(k-1);
            phi(k)=phi(k-1);
        end 
       
    end

endfunction;

